
import lzw

import unittest
import itertools
import random
import struct
import os


# These tests are less interesting than the doctests inside of the lzw
# module itself, they're generally not really unit tests at all;
# rather, they're functional tests written for "dual" subsystems in
# lzw, or else particularly tricky bits and otherwise detected bugs I
# want to be sure don't regress against code changes.

TEST_ROOT = os.path.dirname(__file__)
ENGLISH_FILE = os.path.join(TEST_ROOT, "data", "the_happy_prince.txt")

class TestEncoder(unittest.TestCase):
    
    def setUp(self):
        self.english = None

        with open(ENGLISH_FILE, "rb") as inf:
            self.english = inf.read()

        self.gibberish = b"".join(struct.pack("B", random.randrange(256))
                                  for b in self.english)
            

    def test_readbytes(self):

        realbytes = None

        with open(ENGLISH_FILE, "rb") as inf:
            realbytes = inf.read()

        testbytes = b"".join(lzw.readbytes(ENGLISH_FILE))

        for (old,new) in itertools.izip_longest(realbytes, testbytes):
            self.assertEqual(old, new)


    def test_encoder_packing_assumption(self):
        """
        Our bitpacking scheme relies on assumptions about the size of
        the codebook in relation to the number of codes produced. In
        particular:
        
        - each non-control code emitted by the encoder is FOLLOWED by
          a single addition to the encoder's codebook, or is followed
          by a control code in the output.
         
        - each control code emitted by the encoder is FOLLOWED by a
          reversion of the codebook to it's initial state.

        """
        encoder = lzw.Encoder()
        initial_codesize = encoder.code_size()
        codesize = initial_codesize
        codes_seen = 0

        probablyfailed = False

        for pt in encoder.encode(self.english):
            codesize = encoder.code_size()

            if pt == lzw.CLEAR_CODE:
                codes_seen = 0
                probablyfailed = False
            elif probablyfailed:
                self.fail(probablyfailed)
            else:
                codes_seen = codes_seen + 1
                if initial_codesize + codes_seen != codesize:
                    probablyfailed = "Expected code size {0} but found {1}, Not followed by a control signal"


    def test_encodedecode(self):

        encoder = lzw.Encoder()
        decoder = lzw.Decoder()

        codepoints = encoder.encode(self.english)
        decoded = decoder.decode(codepoints)
        newbytes = b"".join(decoded)

        self.assertEqual(self.english, newbytes)

        encoder = lzw.Encoder()
        decoder = lzw.Decoder()

        codepoints = encoder.encode(self.gibberish)
        decoded = decoder.decode(codepoints)
        newbytes = b"".join(decoded)

        self.assertEqual(self.gibberish, newbytes)

    
    def test_compressdecompress(self):
        english = self.english
        gibberish = self.gibberish

        compressed = lzw.compress(english)
        compressed = [ b for b in compressed ]

        decompressed = b"".join(lzw.decompress(compressed))

        self.assertEqual(english, decompressed)

        compressed = lzw.compress(gibberish)
        compressed = [ b for b in compressed ]

        decompressed = b"".join(lzw.decompress(compressed))
        
        self.assertEqual(gibberish, decompressed)





import lzw
import unittest

import os
import tempfile
import itertools

TEST_ROOT = os.path.dirname(__file__)
BIG_FILE = os.path.join(TEST_ROOT, "data", "library-of-congress-smaller.ppm")
GIANT_FILE = os.path.join(TEST_ROOT, "data", "library-of-congress-photo.ppm")

class TestEncoderSlowly(unittest.TestCase):

    def test_big_file(self):
        self.verify_compressed_file(BIG_FILE)

    def test_giant_file(self):
        self.verify_compressed_file(GIANT_FILE)


    def verify_compressed_file(self, testfile=GIANT_FILE):

        with tempfile.TemporaryFile("w+b") as compressedfile:

            originalsize = 0
            compressedsize = 0
            uncompressedsize = 0

            bigstream = lzw.readbytes(testfile)
            compressed = lzw.compress(bigstream)
            
            for bs in compressed: 
                compressedsize = compressedsize + 1
                compressedfile.write(bs)

            ############################

            compressedfile.flush()
            compressedfile.seek(0)

            checkstream = lzw.readbytes(testfile)
            uncompressed = lzw.decompress(lzw.filebytes(compressedfile))

            for oldbyte, newbyte in itertools.izip_longest(checkstream, uncompressed):
                uncompressedsize = uncompressedsize + 1

                if oldbyte != newbyte:
                    msg = "Corrupted byte at {0}, original {1} != {2}".format(uncompressedsize, oldbyte, newbyte)
                    self.assertEquals(oldbyte, newbyte, msg)





from distutils.core import setup, Command
import unittest
import doctest
from unittest import defaultTestLoader, TextTestRunner

import sys

# BUG. At least we should support 3.0, which means getting
# rid of a bunch of stupid byte-as-string stuff, which is kind
# of awesome.
(major_version, minor_version) = sys.version_info[:2]
if (major_version != 2) or (minor_version < 6):
    raise Exception("LZW currently requires python 2.6")
else:
    import lzw


TEST_MODULE_NAME = "tests.tests"
SLOW_TEST_MODULE_NAME = "tests.slow"
DOC_DIR_NAME = "doc"
MODULES = [ "lzw" ]

class RunTestsCommand(Command):
    """Runs package tests"""
    
    user_options = [('runslow', None, 'also runs the (fairly slow) functional tests')]

    def initialize_options(self):
        self.runslow = False

    def finalize_options(self):
        pass # how on earth is this supposed to work?


    def run(self):
        import lzw
        doctest.testmod(lzw)

        utests = defaultTestLoader.loadTestsFromName(TEST_MODULE_NAME)
        urunner = TextTestRunner(verbosity=2)
        urunner.run(utests)

        if self.runslow:
            utests = defaultTestLoader.loadTestsFromName(SLOW_TEST_MODULE_NAME)
            urunner = TextTestRunner(verbosity=2)
            urunner.run(utests)


class DocCommand(Command):
    """Generates package documentation using epydoc"""

    user_options = []

    def initialize_options(self): pass
    def finalize_options(self): pass

    def run(self):
        # Slightly stupid. Move to sphinx when you can, please.
        import epydoc.cli
        real_argv = sys.argv
        sys.argv = [ "epydoc", "--output", DOC_DIR_NAME, "--no-private" ] + MODULES
        epydoc.cli.cli()
        sys.argv = real_argv


setup(name="lzw",
      description="Low Level, pure python lzw compression/decompression library",

      py_modules=MODULES,
      version=lzw.__version__,
      author=lzw.__author__,
      author_email=lzw.__email__,
      url=lzw.__url__,
      license=lzw.__license__,
      platforms='Python 2.6',
      download_url='http://pypi.python.org/packages/source/l/lzw/lzw-0.01.tar.gz',

      classifiers = [
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Topic :: System :: Archiving",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        ],

      packages = ['lzw'],

      long_description = """
A pure python module for compressing and decompressing streams of
data, built around iterators. Requires python 2.6
""",

      cmdclass = { 
        'test' : RunTestsCommand,
        'doc' : DocCommand,
        },
      )
